
var _ = require('underscore'),
    emitter = require('events').EventEmitter,
    crypto = require('crypto');

module.exports = function (params) {
    if (module.exports.debug) console.log('new model ', params);
    
    params = _.extend({}, params, this);
    this.property = this.model + '_';
    this.collection = [];
    var self = this;
    _.extend(this, new emitter, {
    
        comparator: function (a, b) {
            return a.index - b.index;
        },
        
        add: function (doc) {
            if (module.exports.debug) console.log('adding model ');
            self.create(doc);
        },
        
        create: function (doc) {
            if (module.exports.debug) console.log('creating model ');

            var md5sum = crypto.createHash('md5');
            md5sum.update(String(Math.random()));
            var newid = self.property + md5sum.digest('hex');
            
            doc.id = doc.id || newid;
            doc.index = doc.index || self.collection.length;
            
            if (!doc || !doc.name) return;

            self.collection.push(doc);
            db.hset(self.model, doc.id, JSON.stringify(doc), function (e) {
                if (e) console.dir(e);
                self.emit('add', doc);
                publish.publish(self.model, JSON.stringify(doc));
            });
        },
        
        all: function (fn) {
            if (module.exports.debug) console.log('getting all models ');
            db.hgetall(self.model, function (e, docs) {
                if (e) console.dir(e);
                for (k in docs) {
                    var doc = docs[k];
                    try { 
                        var foo = JSON.parse(doc);
                    } catch (e) { 
                        if (e) console.log(e);
                        var foo = doc;
                    } finally { 
                        self.collection.push(foo);
                    }
                }
                
                // sort the collection
                self.collection = self.collection.sort(self.comparator);
                
                self.collection.forEach(function (doc) {
                    self.emit('add', doc);
                });
            });
        },
        
        read: function (id, fn) {
            if (module.exports.debug) console.log('reading model ');
            db.hget(self.model, id, function (e, doc) {
                if (e) console.dir(e);
                try { 
                    var foo = JSON.parse(doc);
                } catch (e) { 
                    if (e) console.log(e);
                    var foo = doc;
                } finally { 
                    fn(foo);
                }
            });
        },
        
        update: function (doc) {
            if (module.exports.debug) console.log('updating model ');
            if (!doc.name) return;
            
            // remove the item from the collection
            self.collection = _.reject(self.collection, function (itm) {
                return (itm.id === doc.id);
            });
            
            // push the updated item to the collection
            self.collection.push(doc);
            
            db.hset(self.model, doc.id, JSON.stringify(doc), function (e) {
                if (e) console.dir(e);
                self.emit('change', doc);
                publish.publish(self.model, JSON.stringify(doc));
            });
        },
        
        remove: function (ids) {
            if (module.exports.debug) console.log('removing model ');
        
            if (typeof ids === 'undefined') return;
            if (typeof ids === 'string') ids = [].concat(ids);

            self.collection = _.filter(self.collection, function (itm) { 
                if (_.indexOf(ids, itm.id) < 0) {
                    return true;
                } else {
                    db.hdel(self.model, itm.id);
                    itm.remove = true;
                    publish.publish(self.model, JSON.stringify(itm));
                    return false;
                }
            });
        },
        
        sync: function (doc) {
            if (module.exports.debug) console.log('synchronizing model ');
            if (!doc) return;

            if (doc.remove) {
                self.emit('remove', doc);
            } else {
                var ids = _.pluck(self.collection, 'id');
                var foo = _.include(ids, doc.id);
                if (!foo) {
                    self.emit('add', doc);
                    self.initialize && self.initialize(doc);
                }
            }
            
            // remove the item from the collection
            self.collection = _.reject(self.collection, function (itm) {
                return (itm.id === doc.id);
            });
            
            
            if (!doc.remove) {
                self.emit('change', doc);
                    
                // push the updated item to the collection
                self.collection.push(doc);
                
                // sort the collection
                self.collection = self.collection.sort(self.comparator);
            }
            self.emit('change::collection', self.collection);
            
        },
        
    });
    
    pubsub.register(this.model, params.listen, self.sync);
    return this;
};

module.exports.extend = function (params) {
    if (module.exports.debug) console.log('extending model ');
    return function (opts) {
        return module.exports.call(params, opts);
    }
};

var pubsub = {};

pubsub.remove = function (stream, conn) {
    conn.streams.forEach(function (stream) {
        if (stream && conn.id && clients[stream]) {
            delete clients[stream][conn.id];
            conn.streams.splice(conn.streams.indexOf(stream), 1);
        }
        if (stream) {
            subscribe.unsubscribe(stream);
        }
    });
};

pubsub.register = function (stream, conn, callback) {
    conn.streams = conn.streams || [];
    conn.streams.push(stream);
    
    console.log(conn.streams);
    
    if(!clients[stream]) {
        clients[stream] = {};
    }
    clients[stream][conn.id] = callback;
    subscribe.subscribe(stream);
    
    conn.on('end', function () {
        pubsub.remove(stream, conn);
    });
};

