/**  Simple MVC framework 
  *
  *  redis pubsub built in
  *  automatically loads views
  *  automatically loads view rendering engine
  */

var _ = require('underscore'),
hash = require('hashish'),
walk = require('./view.walk');

db = {};
publish = {};
subscribe = {};
clients = {};

exports.Model = require('./model');
exports.View = require('./view');
exports.Controller = require('./controller');
exports.middleware = require('./middleware');

exports.database = {
    type: 'memory',
    args: [],
    connections: []
};

exports['template engine'] = 'ejs';
exports.templates = [];

exports.set = function () {
    if (typeof arguments[0] !== 'string') return;
    var key = arguments[0];
    var value = arguments[1];
    var params = arguments[2];
    switch (key) {
        case 'debug':
            console.log('Tubes in debug mode.');
            exports.debug = value;
            exports.Model.debug = value;
            exports.View.debug = value;
            exports.Controller.debug = value;
            exports.middleware.debug = value;
        break;
        case 'database':
            exports.database.type = value;
            exports.database.args = params;
            if (exports.debug) console.log('setting database', exports.database);
            var file = require(value);
            if (value === 'redis') {
                console.log(params)
                publish = exports.publish = file.createClient.apply({}, params);
                subscribe = exports.subscribe = file.createClient.apply({}, params);
                exports.subscribe.on('message', function(stream, data) {
                    new hash(clients[stream]).forEach(function (emit) {
                        emit.call({}, JSON.parse(data));
                    });
                });
                
                var file = require(value);
                db = exports.db = file.createClient.apply({}, params);
                
            }
            if (value === 'mongodb') exports.db = exports.db.connect(params);
        break;
        case 'template engine':
            if (exports.debug) console.log('setting template engine', value);
            exports['template engine'] = value;
            exports[value] = require(value);
            exports.render = exports[value].render;
        break;
        case 'templates directory':
            if (exports.debug) console.log('setting templates directory', value);
            var dir = arguments[1] || 'views';
            walk('./' + dir, function (e, tpls) {
                tpls = tpls || [];
                if (exports.debug) console.log('Loaded ' + Object.keys(tpls).length + ' views from ' + dir);
                exports.templates = tpls;
            });
        break;
        case 'main':
            if (exports.debug) console.log('setting main controller');
            exports.middleware.main = value;
        break;
        default:
            if (exports.debug) console.log('setting ', key);
            exports[key] = value;
        break;
    }
    return exports;
};

exports.get = function () {
    if (typeof arguments[0] !== 'string') return;
    var key = arguments[0];
    return exports[key];
};

